/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is cmgui_corba_bridge.
 *
 * The Initial Developer of the Original Code is
 * Auckland Uniservices Ltd, Auckland, New Zealand.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "SceneViewerImpl.hpp"
#include <stdlib.h>

void
SceneViewerImpl::close()
{
}

void
SceneViewerImpl::getLookatParameters
(
 CORBA::Double& eyeX,
 CORBA::Double& eyeY,
 CORBA::Double& eyeZ,
 CORBA::Double& lookAtX,
 CORBA::Double& lookAtY,
 CORBA::Double& lookAtZ,
 CORBA::Double& upX,
 CORBA::Double& upY,
 CORBA::Double& upZ
)
{
  if (
      !Cmiss_scene_viewer_get_lookat_parameters
       (
        impl,
        &eyeX, &eyeY, &eyeZ, &lookAtX, &lookAtY, &lookAtZ,
        &upX, &upY, &upZ
       )
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::setLookatParametersNonSkew
(
 CORBA::Double eyeX,
 CORBA::Double eyeY,
 CORBA::Double eyeZ,
 CORBA::Double lookAtX,
 CORBA::Double lookAtY,
 CORBA::Double lookAtZ,
 CORBA::Double upX,
 CORBA::Double upY,
 CORBA::Double upZ
)
{
  if (
      !Cmiss_scene_viewer_set_lookat_parameters_non_skew
       (
        impl,
        eyeX, eyeY, eyeZ, lookAtX, lookAtY, lookAtZ,
        upX, upY, upZ
       )
     )
    throw CMGUIAPI::CmguiError();  
}

void
SceneViewerImpl::getNearAndFarPlane
(
 CORBA::Double& nearPlane,
 CORBA::Double& farPlane
)
{
  if (
      !Cmiss_scene_viewer_get_near_and_far_plane(impl, &nearPlane, &farPlane)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::setNearAndFarPlane
(
 CORBA::Double nearPlane,
 CORBA::Double farPlane
)
{
  if (
      !Cmiss_scene_viewer_set_near_and_far_plane(impl, nearPlane, farPlane)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::getBackgroundColourRGB
(
 CORBA::Double& red,
 CORBA::Double& green,
 CORBA::Double& blue
)
{
  if (
      !Cmiss_scene_viewer_get_background_colour_rgb(impl, &red, &green, &blue)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::setBackgroundColourRGB
(
 CORBA::Double red,
 CORBA::Double green,
 CORBA::Double blue
)
{
  if (
      !Cmiss_scene_viewer_set_background_colour_rgb(impl, red, green, blue)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::setSceneByName(const char* sceneName)
{
  if (
      !Cmiss_scene_viewer_set_scene_by_name
       (impl, const_cast<char*>(sceneName))
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::setOverlaySceneByName(const char* sceneName)
{
  if (
      !Cmiss_scene_viewer_set_overlay_scene_by_name
       (impl, const_cast<char*>(sceneName))
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::setBackgroundTextureByName(const char* textureName)
{
  if (
      !Cmiss_scene_viewer_set_background_texture_by_name
       (impl, const_cast<char*>(textureName))
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::viewAll()
{
  if (
      !Cmiss_scene_viewer_view_all(impl)
     )
    throw CMGUIAPI::CmguiError();  
}

void
SceneViewerImpl::redrawNow()
{
  if (
      !Cmiss_scene_viewer_redraw_now(impl)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::getFreespinTumbleAxis(CORBA::Float& axisX,
                                       CORBA::Float& axisY,
                                       CORBA::Float& axisZ)
{
  CORBA::Float axis[3];
  if (
      !Cmiss_scene_viewer_get_freespin_tumble_axis(impl, axis)
     )
    throw CMGUIAPI::CmguiError();
  axisX = axis[0];
  axisY = axis[1];
  axisZ = axis[2];
}

void
SceneViewerImpl::startFreespin(CORBA::Float tumbleAxisX,
                               CORBA::Float tumbleAxisY,
                               CORBA::Float tumbleAxisZ,
                               CORBA::Double tumbleAngle)
{
  CORBA::Float axis[3] = {tumbleAxisX, tumbleAxisY, tumbleAxisZ};
  if (
      !Cmiss_scene_viewer_start_freespin(impl, axis, tumbleAngle)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::stopAnimations()
{
  if (
      !Cmiss_scene_viewer_stop_animations(impl)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::writeImageToFile
(
 const char* fileName,
 CORBA::Boolean forceOnscreen,
 CORBA::ULong preferredWidth,
 CORBA::ULong preferredHeight,
 CORBA::ULong preferredAntiAlias,
 CORBA::ULong preferredTransparencyLayers
)
{
  if (
      !Cmiss_scene_viewer_write_image_to_file
       (
        impl,
        const_cast<char*>(fileName),
        forceOnscreen,
        preferredWidth,
        preferredHeight,
        preferredAntiAlias,
        preferredTransparencyLayers
       )
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::openSceneEditor()
{
  /* XXX I don't belong here. */
}

void
SceneViewerImpl::setWindowSize
(
 CORBA::ULong width,
 CORBA::ULong height
)
{
#ifdef WIN32
  SetWindowPos(childHWND, HWND_TOP, 0, 0, width, height,
               SWP_NOZORDER | SWP_NOMOVE);
  SendMessage(childHWND, WM_SIZING, 0, 0);
#else
  width = 0;
  height = 0;
  /* XXX fixme */
#endif
}

CORBA::ULong
SceneViewerImpl::projectionMode()
{
  enum Cmiss_scene_viewer_projection_mode csvpm;

  if (!Cmiss_scene_viewer_get_projection_mode(impl, &csvpm))
    throw CMGUIAPI::CmguiError();
  if (csvpm == CMISS_SCENE_VIEWER_PARALLEL)
    return CMGUIAPI::SceneViewer::PROJECTION_PARALLEL;
  return CMGUIAPI::SceneViewer::PROJECTION_PERSPECTIVE;
}

void
SceneViewerImpl::projectionMode(CORBA::ULong projMode)
{
  enum Cmiss_scene_viewer_projection_mode csvpm;

  if (projMode == CMGUIAPI::SceneViewer::PROJECTION_PERSPECTIVE)
    csvpm = CMISS_SCENE_VIEWER_PERSPECTIVE;
  else if (projMode == CMGUIAPI::SceneViewer::PROJECTION_PARALLEL)
    csvpm = CMISS_SCENE_VIEWER_PARALLEL;
  else
    throw CMGUIAPI::CmguiError();

  if (!Cmiss_scene_viewer_set_projection_mode(impl, csvpm))
    throw CMGUIAPI::CmguiError();
}

CORBA::ULong
SceneViewerImpl::viewportMode()
{
  enum Cmiss_scene_viewer_viewport_mode csvvm;
 
  if (!Cmiss_scene_viewer_get_viewport_mode(impl, &csvvm))
    throw CMGUIAPI::CmguiError();

  switch (csvvm)
  {
  case CMISS_SCENE_VIEWER_ABSOLUTE_VIEWPORT:
    return CMGUIAPI::SceneViewer::VIEWPORT_ABSOLUTE;
  case CMISS_SCENE_VIEWER_RELATIVE_VIEWPORT:
    return CMGUIAPI::SceneViewer::VIEWPORT_RELATIVE;
  case CMISS_SCENE_VIEWER_DISTORTING_RELATIVE_VIEWPORT:
    return CMGUIAPI::SceneViewer::VIEWPORT_DISTORTING_RELATIVE;
  }
  throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::viewportMode(CORBA::ULong viewportMode)
{
  enum Cmiss_scene_viewer_viewport_mode csvvm;

  switch (viewportMode)
  {
  case CMGUIAPI::SceneViewer::VIEWPORT_ABSOLUTE:
    csvvm = CMISS_SCENE_VIEWER_ABSOLUTE_VIEWPORT;
    break;
  case CMGUIAPI::SceneViewer::VIEWPORT_RELATIVE:
    csvvm = CMISS_SCENE_VIEWER_RELATIVE_VIEWPORT;
    break;
  case CMGUIAPI::SceneViewer::VIEWPORT_DISTORTING_RELATIVE:
    csvvm = CMISS_SCENE_VIEWER_DISTORTING_RELATIVE_VIEWPORT;
    break;
  default:
    throw CMGUIAPI::CmguiError();
  }
  
  if (!Cmiss_scene_viewer_set_viewport_mode(impl, csvvm))
    throw CMGUIAPI::CmguiError();
}

CORBA::ULong
SceneViewerImpl::transparencyMode()
{
  enum Cmiss_scene_viewer_transparency_mode csvtm;
  if (
      !Cmiss_scene_viewer_get_transparency_mode(impl, &csvtm)
     )
    throw CMGUIAPI::CmguiError();
  switch (csvtm)
  {
  case CMISS_SCENE_VIEWER_FAST_TRANSPARENCY:
    return CMGUIAPI::SceneViewer::TRANSPARENCY_FAST;
  case CMISS_SCENE_VIEWER_SLOW_TRANSPARENCY:
    return CMGUIAPI::SceneViewer::TRANSPARENCY_SLOW;
  case CMISS_SCENE_VIEWER_LAYERED_TRANSPARENCY:
    return CMGUIAPI::SceneViewer::TRANSPARENCY_LAYERED;
  case CMISS_SCENE_VIEWER_ORDER_INDEPENDENT_TRANSPARENCY:
    return CMGUIAPI::SceneViewer::TRANSPARENCY_ORDER_INDEPENDENT;
  }
  throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::transparencyMode(CORBA::ULong transMode)
{
  enum Cmiss_scene_viewer_transparency_mode csvtm;

  switch (transMode)
  {
  case CMGUIAPI::SceneViewer::TRANSPARENCY_FAST:
    csvtm = CMISS_SCENE_VIEWER_FAST_TRANSPARENCY;
    break;
  case CMGUIAPI::SceneViewer::TRANSPARENCY_SLOW:
    csvtm = CMISS_SCENE_VIEWER_SLOW_TRANSPARENCY;
    break;
  case CMGUIAPI::SceneViewer::TRANSPARENCY_LAYERED:
    csvtm = CMISS_SCENE_VIEWER_LAYERED_TRANSPARENCY;
    break;
  case CMGUIAPI::SceneViewer::TRANSPARENCY_ORDER_INDEPENDENT:
    csvtm = CMISS_SCENE_VIEWER_ORDER_INDEPENDENT_TRANSPARENCY;
    break;
  default:
    throw CMGUIAPI::CmguiError();
  }

  if (
      !Cmiss_scene_viewer_set_transparency_mode(impl, csvtm)
     )
    throw CMGUIAPI::CmguiError();
}

CORBA::ULong
SceneViewerImpl::transparencyLayers()
{
  int layerCount;
  
  if (
      !Cmiss_scene_viewer_get_transparency_layers(impl, &layerCount)
     )
    throw CMGUIAPI::CmguiError();

  return (CORBA::ULong)layerCount;
}

void
SceneViewerImpl::transparencyLayers(CORBA::ULong transLayers)
{
  if (
      !Cmiss_scene_viewer_set_transparency_layers(impl, transLayers)
     )
    throw CMGUIAPI::CmguiError();
}

CORBA::Double
SceneViewerImpl::viewAngle()
{
  CORBA::Double angle;
  if (
      !Cmiss_scene_viewer_get_view_angle(impl, &angle)
     )
    throw CMGUIAPI::CmguiError();

  return angle;
}

void
SceneViewerImpl::viewAngle(CORBA::Double angle)
{
  if (!Cmiss_scene_viewer_set_view_angle(impl, angle))
    throw CMGUIAPI::CmguiError();
}

CORBA::ULong
SceneViewerImpl::antialiasMode()
{
  int aaMode;

  if (
      !Cmiss_scene_viewer_get_antialias_mode(impl, &aaMode)
     )
    throw CMGUIAPI::CmguiError();

  return (CORBA::ULong)aaMode;
}

void
SceneViewerImpl::antialiasMode(CORBA::ULong aaMode)
{
  if (
      !Cmiss_scene_viewer_set_antialias_mode(impl, aaMode)
     )
    throw CMGUIAPI::CmguiError();
}

CORBA::Boolean
SceneViewerImpl::perturbLines()
{
  int perturb;
  if (
      !Cmiss_scene_viewer_get_perturb_lines(impl, &perturb)
     )
    throw CMGUIAPI::CmguiError();

  return perturb;
}

void
SceneViewerImpl::perturbLines(CORBA::Boolean pLines)
{
  if (
      !Cmiss_scene_viewer_set_perturb_lines(impl, pLines)
     )
    throw CMGUIAPI::CmguiError();
}

CORBA::Double
SceneViewerImpl::translationRate()
{
  CORBA::Double rate;
  if (
      !Cmiss_scene_viewer_get_translation_rate(impl, &rate)
     )
    throw CMGUIAPI::CmguiError();

  return rate;
}

void
SceneViewerImpl::translationRate(CORBA::Double rate)
{
  if (
      !Cmiss_scene_viewer_set_translation_rate(impl, rate)
     )
    throw CMGUIAPI::CmguiError();
}

CORBA::Double
SceneViewerImpl::tumbleRate()
{
  CORBA::Double rate;
  if (
      !Cmiss_scene_viewer_get_tumble_rate(impl, &rate)
     )
    throw CMGUIAPI::CmguiError();

  return rate;
}

void
SceneViewerImpl::tumbleRate(CORBA::Double rate)
{
  if (
      !Cmiss_scene_viewer_set_tumble_rate(impl, rate)
     )
    throw CMGUIAPI::CmguiError();
}

CORBA::Double
SceneViewerImpl::zoomRate()
{
  CORBA::Double rate;
  if (
      !Cmiss_scene_viewer_get_zoom_rate(impl, &rate)
     )
    throw CMGUIAPI::CmguiError();

  return rate;
}

void
SceneViewerImpl::zoomRate(CORBA::Double rate)
{
  if (
      !Cmiss_scene_viewer_set_zoom_rate(impl, rate)
     )
    throw CMGUIAPI::CmguiError();
}

char*
SceneViewerImpl::sceneName()
{
  char *scene_name_local, *scene_name_return;
  if (
      !Cmiss_scene_viewer_get_scene_name(impl, &scene_name_local)
     )
    throw CMGUIAPI::CmguiError();
  
  /* We have to do this to be CORBA compliant and portable(free/malloc
   * may be separate from the CORBA memory allocators).
   */
  scene_name_return = CORBA::string_dup(scene_name_local);
  free(scene_name_local);
  return scene_name_return;
}

CORBA::Double
SceneViewerImpl::freespinTumbleAngle()
{
  CORBA::Double angle;
  if (!Cmiss_scene_viewer_get_freespin_tumble_angle(impl, &angle))
    throw CMGUIAPI::CmguiError();

  return angle;
}

void
SceneViewerImpl::freespinTumbleAngle(CORBA::Double angle)
{
  if (!Cmiss_scene_viewer_set_freespin_tumble_angle(impl, angle))
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::getViewingVolume
(
 CORBA::Double& left, CORBA::Double& right, CORBA::Double& bottom,
 CORBA::Double& top, CORBA::Double& nearPlane, CORBA::Double& farPlane
)
{
  if (
      !Cmiss_scene_viewer_get_viewing_volume(impl, &left, &right, &bottom,
                                             &top, &nearPlane, &farPlane)
     )
    throw CMGUIAPI::CmguiError();
}

void
SceneViewerImpl::setViewingVolume
(
 CORBA::Double left, CORBA::Double right, CORBA::Double bottom,
 CORBA::Double top, CORBA::Double nearPlane, CORBA::Double farPlane
)
{
  if (
      !Cmiss_scene_viewer_set_viewing_volume(impl, left, right, bottom,
                                             top, nearPlane, farPlane)
     )
    throw CMGUIAPI::CmguiError();
}
