/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 */

#include "mozCellMLRunOnce.h"
#include <nsDirectoryServiceDefs.h>
#include <nsIServiceManagerUtils.h>
#include <nsIPrefService.h>
#include <nsIObserverService.h>
#include <nsIWindowWatcher.h>
#include <stdio.h>
#include <nspr/nspr.h>
#include <plstr.h>
#include <nsString.h>
#include <nsIProperties.h>
#include <nsILocalFile.h>

NS_IMPL_ISUPPORTS1(mozCellMLRunOnce, nsIObserver)

NS_IMETHODIMP
mozCellMLRunOnce::Observe(nsISupports* aSubject,
                          const char* aTopic,
                          const PRUnichar* aData)
{
  if (!PL_strcmp(aTopic, "app-startup"))
  {
    nsCOMPtr<nsIObserverService>
      obsService(do_GetService("@mozilla.org/observer-service;1"));
    // Note the misspelling, it is deliberate(i.e. that is how it appears in
    // Mozilla).
    obsService->AddObserver(this, "startup_user_notifcations", PR_FALSE);
    nsCOMPtr<nsIPrefService> prefservice =
      do_GetService(NS_PREFSERVICE_CONTRACTID);
    nsCOMPtr<nsIPrefBranch> prefbranch;
    prefservice->GetBranch("physiome.mozcellml.", getter_AddRefs(prefbranch));
    PRBool didrunonce = 0;
    nsresult ret = prefbranch->GetIntPref("didrunonce", &didrunonce);
    // On mozilla, but not XRE, we lose the preferences and do it again...
    if (ret == NS_OK && didrunonce == 123)
      return NS_OK;
    ConfigurePreferences(prefbranch);
    prefbranch->SetIntPref("didrunonce", 123);
    return NS_OK;
  }
  else if (!NS_LITERAL_STRING("Creating first window...").Equals(aData))
    return NS_OK;
  nsCOMPtr<nsIPrefService> prefservice =
    do_GetService(NS_PREFSERVICE_CONTRACTID);
  nsCOMPtr<nsIPrefBranch> prefbranch;
  prefservice->GetBranch("physiome.mozcellml.", getter_AddRefs(prefbranch));
  PRBool didrunonce = 0;
  nsresult ret = prefbranch->GetIntPref("didrunonce", &didrunonce);
  if (ret == NS_OK && didrunonce == 123)
    return NS_OK;
  prefbranch->SetIntPref("didrunonce", 123);
  ConfigurePreferences(prefbranch);

  // Open the run-once window(Mozilla only).
  nsCOMPtr<nsIWindowWatcher> ww = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
  nsCOMPtr<nsIDOMWindow> window;
  ww->OpenWindow(NULL, "chrome://mozcellml/content/freshinstall.xul",
                 "_blank", "chrome,dialog=no", nsnull, getter_AddRefs(window));
  return NS_OK;
}

nsresult
mozCellMLRunOnce::FindGCC(nsAString& whichGCC)
{
  // Attempt to locate the component directory...
  nsCOMPtr<nsILocalFile> gccBin;
  nsresult rv;
  nsCOMPtr<nsIProperties>
    dirService(do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv));
  NS_ENSURE_SUCCESS(rv, rv);

  rv = dirService->Get(NS_XPCOM_COMPONENT_DIR,
                       NS_GET_IID(nsIFile),
                       (void**)getter_AddRefs(gccBin));
  NS_ENSURE_SUCCESS(rv, rv);

  gccBin->Append(NS_LITERAL_STRING("extensions"));
  gccBin->Append(NS_LITERAL_STRING("{a0d0a9af-bb3d-46c1-a470-8800b0b3f3e2}"));
  gccBin->Append(NS_LITERAL_STRING("components"));
  gccBin->Append(NS_LITERAL_STRING("gccdist"));
  gccBin->Append(NS_LITERAL_STRING("bin"));
  gccBin->Append(NS_LITERAL_STRING("gcc"));

  PRBool gccExists;
  gccBin->Exists(&gccExists);
  if (gccExists)
  {
    gccBin->GetPath(whichGCC);
    return NS_OK;
  }

  // Now we check out the path...
  char *path_raw = PR_GetEnv("PATH"), *p;
  if (path_raw == nsnull)
    return NS_ERROR_FAILURE;
  for (path_raw = PL_strtok_r(path_raw, ";:", &p);
       path_raw != nsnull;
       path_raw = PL_strtok_r(nsnull, ";:", &p))
  {
    nsAutoString path;
    path.AssignWithConversion(path_raw);
    gccBin->InitWithPath(path);
    gccBin->Append(NS_LITERAL_STRING("gcc"));
    gccBin->Exists(&gccExists);
    if (gccExists)
    {
      gccBin->GetPath(whichGCC);
      return NS_OK;
    }
  }
  return NS_ERROR_FAILURE;
}

nsresult
mozCellMLRunOnce::ConfigurePreferences(nsIPrefBranch* branch)
{
  return NS_OK;
}
