function getenv(name)
{
  var environment =
  Components.classes["@mozilla.org/process/environment;1"]
            .getService(Components.interfaces.nsIEnvironment);
  return environment.get(name);
}

function FindExecutable(name)
{
  var path = getenv("PATH");

  var sep = ":", psep = "/";
  if (navigator.platform.substring(0, 5) == "Win32")
  {
    sep = ";";
    psep = "\\";
    name = name + ".exe"
  }

  // We implicitly add the following to the search path...
  //  1) The local component directory/gcc/bin
  //  2) The profile directory for this extension/gcc/bin
  Components.classes["@mozilla.org/process/environment;1"]
            .getService(Components.interfaces.nsIEnvironment);
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components
                                         .interfaces
                                         .nsIProperties);
  var comsd = dirService.get("ComsD", Components.interfaces.nsIFile);
  comsd.append("gcc");
  comsd.append("bin");
  if (comsd.exists())
    path = (comsd.path) + sep + path;
  var comsd = dirService.get("ProfD", Components.interfaces.nsIFile);
  comsd.append("extensions");
  comsd.append("{a0d0a9af-bb3d-46c1-a470-8800b0b3f3e2}");
  comsd.append("components");
  comsd.append("gcc");
  comsd.append("bin");
  if (comsd.exists())
    path = (comsd.path) + sep + path;

  var idx;
  while ((idx = path.indexOf(sep)) != -1)
  {
    try
    {
      var subpath = path.substring(0, idx);
      path = path.substring(idx + 1);
      var fullpath = subpath + psep + name;
      var lf = Components.classes["@mozilla.org/file/local;1"]
                         .createInstance(Components.interfaces.nsILocalFile);
      lf.initWithPath(fullpath);
      if (lf.exists())
        return fullpath;
    }
    catch (e)
    {
    }
  }
  return null;
}

function GetVariableTreePref()
{
  var prefservice =
  Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService);
  var cellBranch = prefservice.getBranch("physiome.mozcellml.");
  try
  {
    return cellBranch.getBoolPref("useTreeLayout");
  }
  catch (e)
  {
    cellBranch.setBoolPref("useTreeLayout", true);
    return true;
  }
}

function GetCompiler()
{
  var prefservice =
  Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService);
  var cellBranch = prefservice.getBranch("physiome.mozcellml.");
  var compline, cc;
  try
  {
    compline = cellBranch.getCharPref("compiler");
    if (compline == "" || compline == null)
      throw "Compiler is empty";
  }
  catch (e)
  {
    cc = FindExecutable("gcc");
    if (cc == null)
    {
      cc = "/usr/bin/gcc";
    }
    else // it might have spaces in it...
      cc = "\"" + cc + "\"";

    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                               .getService(Components
                                           .interfaces
                                           .nsIProperties);
    var components = dirService.get("ComsD", Components.interfaces.nsIFile);
    var compTest = components.clone();
    compTest.append("DoIntegration.I");
    if (!compTest.exists())
    {
      components = dirService.get("ProfD", Components.interfaces.nsIFile);
      compTest = components.clone();
      compTest.append("DoIntegration.I");
      if (!compTest.exists())
      {
        components.append("extensions");
        components.append("{a0d0a9af-bb3d-46c1-a470-8800b0b3f3e2}");
        components.append("components");
      }
    }
    var includePath = "-I\"" + components.path + "\"";

    var compline = cc + " -O0 " + includePath + " -lnspr4";
    if (navigator.platform.substring(0, 5) == "Win32")
      compline = compline + " -Wl,--subsystem -Wl,windows";
    else
      compline = compline + " -lm";
    cellBranch.setCharPref("compiler", compline);
  }
  return compline;
}

function IntegrationScaleWatcher(npoints)
{
  this.curpoint = 0;
  this.npoints = npoints + 1;
  this.launchDoc = document;
  return this;
}

IntegrationScaleWatcher.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.mozCellMLIScaleWatcher) ||
        aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.CmguiIApplicationBootstrap)
       )
    {
      return this;
    }
    throw Components.results.NS_NOINTERFACE;
  },

  cmguiReady: function(cdf)
  {
    this.commandData = cdf.createCommandData();

    window.openDialog(
                      "chrome://mozcellml/content/integrationresult.xul",
                      "_blank",
                      "chrome,dialog=no",
                      this
                     );
  },

  activateIntegration: function()
  {
    this.live = true;
    if (!this.closed)
    {
      SetupZincForNewIntegrator(this);
      ChangeIntegratorButtons(this.launchDoc, true, true);
      this.integrator.start
      (
        window.model, this.startx, this.stopx, this.gridx,
        this.abserr, this.relerr, this.maxstep
      );
      var statusBar = this.launchDoc.getElementById('status-bar-display');
      var strb = this.launchDoc.getElementById("mozcellml-string-bundle");
      statusBar.label = strb.getString("integrationStarted");
    }
  },

  scaleChanged: function(variable, newMaxAbs)
  {
    ComputeCmguiScale(this, variable, newMaxAbs);
  },

  progress: function(percent)
  {
    if (this.meter != null)
      this.meter.setAttribute("value", percent);
  },

  done: function()
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("integrationDone");
    if (window.integrator != null)
      ChangeIntegratorButtons(document, true, false);
    this.live = false;
    if (this.meter != null)
      this.meter.setAttribute("value", "100%");
  },

  halted: function()
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("integrationHalted");
    this.live = false;
    if (window.integrator != null)
      ChangeIntegratorButtons(document, true, false);
  },

  failed: function(emsg)
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    var idx = emsg.indexOf('/');
    if (idx == -1)
    {
      var err = strb.getFormattedString("integrationFailedReason", [strb.getString(emsg)]);
      statusBar.label = err;
      alert(err);
      this.live = false;
      if (window.integrator != null)
        ChangeIntegratorButtons(document, true, false);
    }
    else
    {
      var emsg1 = emsg.substring(0, idx);
      var emsg2 = emsg.substring(idx + 1);
      var err = strb.getFormattedString("integrationFailedReason", [strb.getFormattedString(emsg1, [emsg2])]);
      statusBar.label = err;
      alert(err);
      this.live = false;
      if (window.integrator != null)
        ChangeIntegratorButtons(document, true, false);
    }
  },

  live: false,
  closed: false
};
