/**
 * Get a structure containing the overrides for each type of window.
 * @return A structure containing the overrides for each type of window.
 */
function GetOverrides()
{
  var w_ors = {
    general: GeneralWindowOverride,
    loadmodel: LoadModelOverride
  };

  return w_ors;
}

/**
 * Produces a new general virtual window.
 */
function CreateNewWindow()
{
  window.lastwinid++;
  var w = new VWindow(window.windowarea, "Window #" + window.lastwinid);
  w.strb = document.getElementById("mozcellml-string-bundle");
  w.setPosition(20, 20, 20, 20);
  w.loadContent(document, "mozcellml-string-bundle");
  w.loadContent(document, "content-pane");
  w.setOverride(GetOverrides(), "general");
  w.setupForContent(w.loadedContent);
  ChangeGeneralView("variables",
                    w.contentArea.getElementsByTagName("vbox").item(0));

  var selResource;
  try
  {
    var treeidx = window.mt.view.selection.currentIndex;
    var builder = window.mt.
                  view.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
    var selResource = builder.getResourceAtIndex(treeidx);
  }
  catch (e)
  {
    return -1;
  }

  var targ = window.active_source.GetTarget(selResource, window.inst_id_r, true);
  var iv = targ.QueryInterface(Components.interfaces.nsIRDFInt).Value;

  var inst = window.model_table[iv];
  w.updateModel(inst);
}

/**
 * Produces a new LoadModel virtual window.
 */
function LoadModelWindow()
{
  var w = new VWindow(window.windowarea, "Load a model");
  w.strb = document.getElementById("mozcellml-string-bundle");
  w.setPosition(10, 10, 10, 10);
  w.setSize(800, 100);
  w.loadContent(document, "mozcellml-string-bundle");
  w.loadContent(document, "loadnew-pane");
  w.setOverride(GetOverrides(), "loadmodel");
}

/**
 * Asks the user if they want to start a new workspace, and does so if they
 * agree.
 */
function NewWorkspace()
{
  if (!confirm("Close this workspace and create new workspace?"))
    return;

  DoNewWorkspace();
}

/**
 * Starts a new workspace without prompting the user.
 */
function DoNewWorkspace()
{
  var divs = window.windowarea.document.documentElement.getElementsByTagName("div");
  for (i = 0; i < divs.length; i++)
  {
    var div = divs.item(i);
    if (div.vwindow != null)
    {
      div.vwindow.performClose();
      i--;
    }
  }

  if (window.xml_models != null)
  {
    var m;
    for (m in  window.xml_models)
    {
      m = window.xml_models[m];
      if (m.integrator != null)
      {
        m.integrator.destroy();
        m.integrator = null;
      }
      m.cellml = null;
    }
  }
  if (window.model_table != null)
  {
    for (m in window.model_table)
    {
      m = window.model_table[m];
      if (m.cmgui != null)
      {
        m.cmgui = null
      }
    }
  }
  window.last_internid = null;
  window.lastwinid = 0;
  window.general_windows = [];
  window.windowarea.windows = [];
  window.windowarea.windowCounter = 1;
  window.windowarea.windowId = 1;
  
  window.xml_models = [];
  window.model_table = [];
  window.last_instance_id = 0;
  // Clear out the models list...
  window.rdfsource_latest = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                                      .createInstance(Components.interfaces.nsIRDFDataSource);
  window.rdfsource_nested = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                                      .createInstance(Components.interfaces.nsIRDFDataSource);
  window.rdfsource_flat = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                                    .createInstance(Components.interfaces.nsIRDFDataSource);
  var rdfmodelroot = window.rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#model-root");
  window.models_latest = window.rdfcu.MakeSeq(window.rdfsource_latest, rdfmodelroot);
  window.models_nested = window.rdfcu.MakeSeq(window.rdfsource_nested, rdfmodelroot);
  window.models_flat = window.rdfcu.MakeSeq(window.rdfsource_flat, rdfmodelroot);

  window.active_view = "flat";

  if (window.active_source != null)
    window.mt.database.RemoveDataSource(window.active_source);
  window.active_source = window["rdfsource_" + window.active_view];
  window.mt.database.AddDataSource(window.active_source);
  window.mt.builder.rebuild();
}

/**
 * Prompts the user for a filename, and saves the workspace to that filename.
 */
function SaveWorkspace()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  try
  {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(nsIFilePicker);
    fp.init(window, strb.getString("chooseWorkspaceSaveFilename"),
            nsIFilePicker.modeSave);
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                               .getService(Components
                                           .interfaces
                                           .nsIProperties);
    var homedir = dirService.get("Home", Components.interfaces.nsIFile);
    fp.displayDirectory = homedir;
    fp.defaultString = ".cellml_workspace.xml";
    fp.appendFilter("XML", "*.xml");
    fp.appendFilter(strb.getString("allFiles"), "*");
    if (fp.show() == nsIFilePicker.returnCancel)
      return;
  }
  catch (ex)
  {
  }
  DoSaveWorkspace(fp.file.path);
}

/**
 * Saves the workspace to the specified filename.
 */
function DoSaveWorkspace(filename)
{
  var divs = window.windowarea.document.documentElement.getElementsByTagName("div");
  var wsd = document.implementation.createDocument
  (
    "http://www.cellml.org/tools/mozCellML/cellml_workspace#1.0",
    "cellml_workspace",
    null
  );
  var we = wsd.createElement("windows");
  var gs = wsd.documentElement;
  gs.setAttribute("modeltreestate", StringifyTreeState(window.mt));
  gs.setAttribute("lastwinid", window.lastwinid);
  if (window.windowarea.windowCounter == null)
    window.windowarea.windowCounter = 1;
  gs.setAttribute("lastwinz", window.windowarea.windowCounter);
  gs.setAttribute("lastinstanceid", window.last_instance_id);
  gs.setAttribute("active_view", window.active_view);
  gs.appendChild(we);
  for (i = 0; i < divs.length; i++)
  {
    var div = divs.item(i);
    if (div.vwindow != null)
    {
      var win_el = wsd.createElement("window");
      div.vwindow.saveState(wsd, win_el);
      we.appendChild(win_el);
    }
  }
  var instid;
  for (instid in window.model_table)
  {
    model = window.model_table[instid];
    var me = wsd.createElement("modeldescription");
    
    if (model.xmlmi == model)
    {
      me.setAttribute("type", "fromxml");
      me.setAttribute("instance_id", model.instance_id);
      if (model.lastTreeState != null)
        me.setAttribute("lastTreeState", model.lastTreeState);
      if (model.lastTreeStateResults != null)
        me.setAttribute("lastTreeStateResults", model.lastTreeStateResults);
      if (model.lastImportantState != null)
        me.setAttribute("lastImportantState", model.lastImportantState);
      if (model.last_graph_id != null)
        me.setAttribute("lastGraphID", model.last_graph_id);
      if (model.drawTree)
        me.setAttribute("drawTree", "true");
      if (model.hideUnimportant)
        me.setAttribute("hideUnimportant", "true");
      if (model.resultTime)
        me.setAttribute("resultTime", model.resultTime);
      if (model.selectedGraph)
        me.setAttribute("selectedGraph", model.selectedGraph);

      var md = wsd.createElement("cellml");
      model.cellml.QueryInterface(Components.interfaces.mozICellMLLocal)
           .populateModelDescription(md);
      me.appendChild(md);

      // XML model instances also have associated graphs...
      var gid;
      if (model.graphs == null)
        model.graphs = [];
      for (gid in model.graphs)
      {
        var gr = model.graphs[gid];
        var gre = wsd.createElement("graph");
        gre.setAttribute("name", gr.name);
        gre.setAttribute("id", gr.id);
        gre.setAttribute("last_trace_id", gr.last_trace_id);
        if (gr.left != null)
        {
          gre.setAttribute("left", gr.left);
          gre.setAttribute("right", gr.right);
          gre.setAttribute("top", gr.top);
          gre.setAttribute("bottom", gr.bottom);
          gre.setAttribute("eyeX", gr.eyeX);
          gre.setAttribute("eyeY", gr.eyeY);
          gre.setAttribute("eyeZ", gr.eyeZ);
          gre.setAttribute("lookatX", gr.lookatX);
          gre.setAttribute("lookatY", gr.lookatY);
          gre.setAttribute("lookatZ", gr.lookatZ);
        }
        if (gr.selectedTrace != null)
          gre.setAttribute("selectedTrace", gr.selectedTrace);
        if (gr.xtreeState != null)
          gre.setAttribute("xtreeState", gr.xtreeState);
        if (gr.ytreeState != null)
          gre.setAttribute("ytreeState", gr.ytreeState);
        var trid;
        for (trid in gr.traces)
        {
          var tre = wsd.createElement("trace");
          var tr = gr.traces[trid];
          tre.setAttribute("colour", tr.colour);
          tre.setAttribute("ximport", tr.xvarlev.impno);
          tre.setAttribute("xcomponent", tr.xvarlev.compName);
          tre.setAttribute("xvariable", tr.xvarlev.name);
          tre.setAttribute("yimport", tr.yvarlev.impno);
          tre.setAttribute("ycomponent", tr.yvarlev.compName);
          tre.setAttribute("yvariable", tr.yvarlev.name);
          gre.appendChild(tre);
        }
        me.appendChild(gre);
      }
    }
    else
    {
      me.setAttribute("type", "submodel");
      me.setAttribute("parentInstance", model.parentInstance.instance_id);
      me.setAttribute("name", model.name);
      me.setAttribute("instance_id", model.instance_id);
      me.setAttribute("dirty", model.dirty);
      me.setAttribute("startx", model.startx);
      me.setAttribute("stopx", model.stopx);
      me.setAttribute("gridx", model.gridx);
      me.setAttribute("relError", model.relError);
      me.setAttribute("absError", model.absError);
      me.setAttribute("maxStep", model.maxStep);
      if (model.initialValues != null)
      {
        var i;
        for (i in model.initialValues)
        {
          var ivi = model.initialValues[i], k1;
          for (k1 in ivi)
          {
            var ivk1 = ivi[k1], k2;
            for (k2 in ivk1)
            {
              var p = wsd.createElement("param");
              var ivk2 = ivk1[k2];
              p.setAttribute("import", i);
              p.setAttribute("component", k1);
              p.setAttribute("variable", k2);
              p.setAttribute("value", ivk2.value);
              md.appendChild(p);
            }
          }
        }
      }
    }
    gs.appendChild(me);
  }

  var strb = document.getElementById("mozcellml-string-bundle");
  var file = Components.classes["@mozilla.org/file/local;1"]
                    .createInstance(Components.interfaces.nsILocalFile);
  try
  {
    file.initWithPath(filename);
  }
  catch (e)
  {
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("cantSaveToPath");
    return;
  }
  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                    .createInstance(Components.interfaces.nsIFileOutputStream);
  var serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"]
                    .createInstance(Components.interfaces.nsIDOMSerializer);
  stream.init(
              file,
              /* PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE */
              42,
              /*0644*/420,
              0
             );
  serializer.serializeToStream(wsd, stream, "UTF8");
  GetStatusBar().setAttribute("label", "Workspace saved");
}

/**
 * Callback for when the workspace has finished loading.
 */
function WorkspaceLoadComplete(event)
{
  GetStatusBar().setAttribute("label", "Workspace loaded.");
  var req = event.target.responseXML;

  window.lastwinid = req.documentElement.getAttribute("lastwinid");
  window.windowarea.windowCounter = req.documentElement.getAttribute("lastwinz");
  window.last_instance_id = req.documentElement.getAttribute("lastinstanceid");
  window.active_view = req.documentElement.getAttribute("active_view");

  document.getElementById("viewmenu").value = window.active_view;
  window.mt.database.RemoveDataSource(window.active_source, true);
  window.active_source = window["rdfsource_" + window.active_view];
  if (window.active_source != null)
    window.mt.database.AddDataSource(window.active_source);

  window.xml_models = [];
  var xd = req.documentElement.getElementsByTagName("modeldescription");
  var i;
  for (i = 0; i < xd.length; i++)
  {
    var md = xd.item(i);
    var model = Components.classes["@physiome.org/mozCellML;1"]
                          .createInstance(Components.interfaces.mozICellMLLocal);
    if (md.getAttribute("type") == "fromxml")
    {
      var cm = md.getElementsByTagName("cellml").item(0);
      model.restoreModelDescription(cm);
      model.initialise();
      var inst = new XMLModelInstance(model.title, model);
      inst.instance_id = md.getAttribute("instance_id");
      if (window.model_table == null)
        window.model_table = [];

      inst.win = window;
      window.model_table[inst.instance_id] = inst;
      inst.setRDF(window.rdfserv, window.model_name_r, window.inst_id_r,
                  window.rdfsource_latest, window.rdfsource_flat,
                  window.rdfsource_nested, window.rdfcu);
      inst.addToAllContainers(window.models_latest, window.models_flat,
                              window.models_nested);
      inst.lastTreeState = md.getAttribute("lastTreeState");
      inst.lastTreeStateResults = md.getAttribute("lastTreeStateResults");
      inst.drawTree = md.hasAttribute("drawTree");
      inst.hideUnimportant = md.hasAttribute("hideUnimportant");
      inst.lastImportantState = md.getAttribute("lastImportantState");
      inst.last_graph_id = md.getAttribute("lastGraphID");
      inst.resultTime = md.getAttribute("resultTime");
      inst.selectedGraph = md.getAttribute("selectedGraph");
      inst.graphs = [];
      var graphList = md.getElementsByTagName("graph");
      var j;
      for (j = 0; j < graphList.length; j++)
      {
        var gre = graphList.item(j);
        var gr = new Graph(gre.getAttribute("name"), gre.getAttribute("id"));
        inst.graphs[gr.id] = gr;
        gr.last_trace_id = gre.getAttribute("last_trace_id");
        if (gre.hasAttribute("left"))
        {
          gr.left = parseFloat(gre.getAttribute("left"));
          gr.right = parseFloat(gre.getAttribute("right"));
          gr.top = parseFloat(gre.getAttribute("top"));
          gr.bottom = parseFloat(gre.getAttribute("bottom"));
          gr.eyeX = parseFloat(gre.getAttribute("eyeX"));
          gr.eyeY = parseFloat(gre.getAttribute("eyeY"));
          gr.eyeZ = parseFloat(gre.getAttribute("eyeZ"));
          gr.lookatX = parseFloat(gre.getAttribute("lookatX"));
          gr.lookatY = parseFloat(gre.getAttribute("lookatY"));
          gr.lookatZ = parseFloat(gre.getAttribute("lookatZ"));
        }
        gr.selectedTrace = parseInt(gre.getAttribute("selectedTrace"));
        gr.xtreeState = gre.getAttribute("xtreeState");
        gr.ytreeState = gre.getAttribute("ytreeState");

        // Next add the traces...
        var traceList = md.getElementsByTagName("trace");
        var k;
        for (k = 0; k < traceList.length; k++)
        {
          var tre = traceList.item(k);
          var xv = new AxisVariable(
             inst.cellml.getVariable(
              tre.getAttribute("ximport"),
              tre.getAttribute("xcomponent"),
              tre.getAttribute("xvariable")
             )
            );
          var yv = new AxisVariable(
             inst.cellml.getVariable(
              tre.getAttribute("yimport"),
              tre.getAttribute("ycomponent"),
              tre.getAttribute("yvariable")
             )
            );
          var tr = new GraphTrace(xv, yv);
          tr.colour = tre.getAttribute("colour");
          gr.traces[k] = tr;
        }
      }
      window.xml_models.push(inst);
    }
    else // its a submodel...
    {
      var parInst = md.getAttribute("parentInstance");
      var inst = window.model_table[parInst].
                        cloneInstance(md.getAttribute("name"),
                                      md.getAttribute("instance_id"));
      inst.dirty = (md.getAttribute("dirty") == "true");
      inst.startx = parseFloat(md.getAttribute("startx"));
      inst.stopx = parseFloat(md.getAttribute("stopx"));
      inst.gridx = parseFloat(md.getAttribute("gridx"));
      inst.relError = parseFloat(md.getAttribute("relError"));
      inst.absError = parseFloat(md.getAttribute("absError"));
      inst.maxStep = parseFloat(md.getAttribute("maxStep"));

      // We now have to load the parameters...
      var pl = md.getElementsByTagName("param");
      var j;
      for (j = 0; j < pl.length; j++)
      {
        var plj = pl.item(j);
        var impno, compname, varname, value;
        impno = plj.getAttribute("import");
        compname = plj.getAttribute("component");
        varname = plj.getAttribute("variable");
        value = plj.getAttribute("value");
        inst.setInitialValue(impno, compname, varname, value);
      }
    }
  }

  var mts = req.documentElement.getAttribute("modeltreestate");
  var os = window.mt.getAttribute("onselect");
  window.mt.removeAttribute("onselect");
  if (mts != null)
    LoadTreeState(window.mt, mts);
  window.mt.setAttribute("onselect", os);

  var wn = req.documentElement.getElementsByTagName("windows").item(0);
  var wc = wn.getElementsByTagName("window");

  for (i = 0; i < wc.length; i++)
  {
    var winnode = wc.item(i);
    var win = new VWindow(window.windowarea, "");
    win.strb = document.getElementById("mozcellml-string-bundle");
    win.restoreState(GetOverrides(), document, winnode);
  }

  // If there is an extra model, load it too...
  if (window.autoloadModel)
  {
    window.autoloadModel = window.autoloadModel.
           QueryInterface(Components.interfaces.mozICellMLLocal);
    window.page.displayModel(window.autoloadModel);
    window.autoloadModel = null;
  }
}

/**
 * Callback for when the workspace failed to load.
 */
function WorkspaceLoadFailed(event)
{
  GetStatusBar().setAttribute("label", "Workspace load failed. Using default workspace.");
  DoNewWorkspace();
  CreateNewWindow();
  LoadModelWindow();

  // If there is an extra model, load it too...
  if (window.autoloadModel)
  {
    window.autoloadModel = window.autoloadModel.
           QueryInterface(Components.interfaces.mozICellMLLocal);
    window.page.displayModel(window.autoloadModel);
    window.autoloadModel = null;
  }
}

/**
 * Attempts to load a workspace from a URL.
 * @param url A string containing the URL to load.
 */
function AttemptWorkspaceLoad(url)
{
  var req = new XMLHttpRequest();
  try
  {
    req.open("GET", url, true);
    req.overrideMimeType('text/xml');
    req.onload = WorkspaceLoadComplete;
    req.onerror = WorkspaceLoadFailed;
    req.send(null);
  }
  catch (e)
  {
    WorkspaceLoadFailed();
  }
}

/**
 * Prompts the user for a file from which to load their workspace, and then
 * loads the workspace from that file.
 * Note It will firstly try to save the current workspace, which the new
 *      workspace will replace.
 */
function LoadWorkspace()
{
  SaveWorkspace();
  var strb = document.getElementById("mozcellml-string-bundle");
  const nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
                     .createInstance(nsIFilePicker);
  fp.init(window, strb.getString("chooseWorkspaceLoadFilename"),
          nsIFilePicker.modeOpen);
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components
                                         .interfaces
                                         .nsIProperties);
  var homedir = dirService.get("Home", Components.interfaces.nsIFile);
  fp.displayDirectory = homedir;
  fp.defaultString = ".cellml_workspace.xml";
  fp.appendFilter("XML", "*.xml");
  fp.appendFilter(strb.getString("allFiles"), "*");
  if (fp.show() == nsIFilePicker.returnCancel)
    return;
  DoNewWorkspace();

  AttemptWorkspaceLoad("file:///" + fp.file.path);
}

/**
 * Set up the unified CellML environment.
 */
function SetupUnified()
{
  if (window.alreadyLoaded != null)
    return;
  window.alreadyLoaded = 1;

  window.windowarea = window.frames[0];

  if (window.windowarea == null)
    return;

  window.lastwinid = 0;

  // Load the preparer first, to make sure the paths are right...
  Components.classes["@physiome.org/mozCellMLPreparer;1"]
    .createInstance(Components.interfaces.nsISupports);

  // We have two choices:
  // 1) Load .cellml_workspace, or
  // 2) Create the default layout.
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components.interfaces.nsIProperties);
    var homedir = dirService.get("Home", Components.interfaces.nsIFile);
  homedir.append(".cellml_workspace.xml");

  window.page = new UnifiedPage(window);
  var srv = Components.classes["@physiome.org/mozCellML/UIService;1"]
            .getService(Components.interfaces.mozICellMLUIService);
  window.uiserv = srv;
  srv.setUI(window.page);

  window.mt = document.getElementById("model-tree");
  window.rdfsource_latest = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                                      .createInstance(Components.interfaces.nsIRDFDataSource);
  window.rdfsource_nested = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                                      .createInstance(Components.interfaces.nsIRDFDataSource);
  window.rdfsource_flat = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                                    .createInstance(Components.interfaces.nsIRDFDataSource);
  window.rdfserv = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                          .getService(Components.interfaces.nsIRDFService);

  window.rdfcu = Components.classes['@mozilla.org/rdf/container-utils;1']
                           .getService(Components.interfaces.nsIRDFContainerUtils);
    
  var rdfmodelroot = window.rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#model-root");
  window.models_latest = window.rdfcu.MakeSeq(window.rdfsource_latest, rdfmodelroot);
  window.models_nested = window.rdfcu.MakeSeq(window.rdfsource_nested, rdfmodelroot);
  window.models_flat = window.rdfcu.MakeSeq(window.rdfsource_flat, rdfmodelroot);

  window.model_name_r = window.rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#model-name");
  window.model_props_r = window.rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#model-properties");
  window.inst_id_r = window.rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#model-instance-id");
  window.rdfpropdflt = window.rdfserv.GetLiteral("defaultunified");

  if (window.active_view == null)
    window.active_view = "flat";

  window.active_source = window["rdfsource_" + window.active_view];
  window.mt.database.AddDataSource(window.active_source);

  if (window.arguments && window.arguments.length > 0)
    window.autoloadModel = window.arguments[0];

  AttemptWorkspaceLoad("file:///" + homedir.path);
}

/**
 * Callback for after the page has finished closing.
 */
function CloseUnified()
{
  window.uiserv.clearUI();
  window.uiserv = null;
  for (m in  window.xml_models)
  {
    if (m.integrator != null)
    {
      m.integrator.destroy()
      m.integrator = null;
    }
  }
  // Give the cleanup some help or it never seems to happen.
  window.xml_models = null;
  window.model_table = null;
}

/**
 * Callback for after a model has been selected.
 */
function ModelSelected(e)
{
  var selResource;
  try
  {
    var treeidx = window.mt.view.selection.currentIndex;
    var builder = window.mt.
                  view.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
    var selResource = builder.getResourceAtIndex(treeidx);
  }
  catch (e)
  {
    return -1;
  }

  var targ = window.active_source.GetTarget(selResource, window.inst_id_r, true);
  var iv = targ.QueryInterface(Components.interfaces.nsIRDFInt).Value;

  var inst = window.model_table[iv];

  // Now we have to notify all the windows that we were selected.
  var i;
  for (i = 0; i < window.general_windows.length; i++)
    if (window.general_windows[i] != null)
      window.general_windows[i].updateModel(inst);
}

function ModelSwitch(win, newmod)
{
  var selResource;

  win.mt = win.mt.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  var i;
  for (i = 0; i < win.mt.view.rowCount; i++)
    if (!win.mt.view.isContainerOpen(i))
      win.mt.view.toggleOpenState(i);

  var selIndex;
  try
  {
    var builder = win.mt.
                  view.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
    selIndex = builder.getIndexOfResource(newmod.self_resource);
  }
  catch (e)
  {
    return -1;
  }
  win.mt.view.selection.select(selIndex);
  win.mt.view.selection.currentIndex = selIndex;
}

/**
 * Called whenever the tree view for models is changed.
 * @param view The new view.
 */
function ChangeModelTreeView(view)
{
  if (window.active_source != null)
    window.mt.database.RemoveDataSource(window.active_source, true);
  window.active_view = view;
  window.active_source = window["rdfsource_" + view];
  if (window.active_source != null)
    window.mt.database.AddDataSource(window.active_source);
  window.mt.builder.rebuild();
}

function ErrorMessage(m)
{
  this.m = m;
}

ErrorMessage.prototype =
{
  QueryInterface: function(aIID)
  {
    if (aIID.equals(Components.interfaces.mozILoadError) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  getError: function(b)
  {
    return this.m;
  }
};

/**
 * The constructor for the UnifiedPage class.
 * UnifiedPage is a class which can hook into the mozICellMLUIService service
 * to receive all models loaded on the system.
 */
function UnifiedPage(w)
{
  this.window = w;
  this.models = [];
  return this;
};

/**
 * The prototype for the UnifiedPage class.
 */
UnifiedPage.prototype =
{
  /**
   * QueryInterface for UnifiedPage.
   */
  QueryInterface: function(iid)
  {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.mozICellMLUIService))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  /**
   * Not implemented, as this is a terminal UIService.
   */
  setUI: function(pt)
  {
  },

  /**
   * Not implemented, as this is a terminal UIService.
   */
  clearUI: function()
  {
  },

  /**
   * Callback for whenever the system has finished loading a mozICellML.
   * @param model The mozICellML which has just been loaded.
   */
  displayModel: function(model)
  {
    var ts = Components.classes["@physiome.org/TranslationSet;1"]
                       .createInstance(Components.interfaces.mozITranslationSetLocal);
    ts.bundle = this.window.document.getElementById("mozcellml-string-bundle").stringBundle;
    ts = ts.QueryInterface(Components.interfaces.mozITranslationSet);
    model.setStringBundle(ts);
    try
    {
      model.initialise();
    }
    catch (ex)
    {
    }
    // If there are errors, display them now, and bail. Otherwise, it can create
    // unexpected values later.
    var v = model.checkValidity();
    if (v != "")
    {
      openDialog(
                 "chrome://mozcellml/content/loaderror.xul",
                 "_blank",
                 "chrome,dialog=no",
                 new ErrorMessage(v)
                );
      return;
    }

    // Next try warnings...
    v = model.getWarnings();
    if (v != "")
    {
      openDialog(
                 "chrome://mozcellml/content/loaderror.xul",
                 "_blank",
                 "chrome,dialog=no",
                 new ErrorMessage(v)
                );
    }

    var inst = new XMLModelInstance(model.title, model);
    // We now need to instruct the model to populate the trees.
    inst.addToModelTable(this.window);
    inst.setRDF(this.window.rdfserv, this.window.model_name_r, this.window.inst_id_r,
                this.window.rdfsource_latest, this.window.rdfsource_flat,
                this.window.rdfsource_nested, this.window.rdfcu);
    inst.addToAllContainers(this.window.models_latest, this.window.models_flat,
                            this.window.models_nested);
    this.window.xml_models.push(inst);

    GetStatusBar().setAttribute("label", "Document loaded");
  }
};

/**
 * A function that can be called from anywhere to get hold of the global status
 * bar.
 */
function GetStatusBar()
{
  return window.top.document.getElementById("status-bar-display");
}
