/**
 * Initialise the Variables general window view.
 * @param content The content area.
 */
function InitVariables(content)
{
  this.activeModel = null;
  this.modelNameLabel = content.getElementsByTagName("label").item(1);
  this.tree = content.getElementsByTagName("tree").item(0);
  var hb1 = content.getElementsByTagName("hbox").item(1);
  var cbl = hb1.getElementsByTagName("checkbox");
  this.hide_unimportant = cbl.item(0);
  this.draw_tree = cbl.item(1);
  this.var_important = cbl.item(2);
  var hb2 = hb1.getElementsByTagName("hbox").item(0);
  this.ivbox = hb2.getElementsByTagName("input").item(0);
  content.vwin = this;
}

/**
 * Save any information about this window. The information saved here should be
 * information which is not preserved across a content type switch, but which
 * should be preserved across a load and save when the content type is not
 * changed.
 * @param node A DOM node onto which information should be stored.
 */
function VariablesSaveContent(node)
{
  node.setAttribute("treeState", StringifyTreeState(this.tree));
  if (this.modlev)
    node.setAttribute("importantState", this.modlev.stringifyImportantState());
}

/**
 * Restore any information which was saved to node in VariablesSaveContent.
 * @param node A DOM node onto which information should be stored.
 */
function VariablesRestoreContent(node)
{
  var ts = node.getAttribute("treeState");
  if (ts != null)
    LoadTreeState(this.tree, ts);
  var imps = node.getAttribute("importantState");
  if (this.modlev && imps)
    this.modlev.restoreImportantState(imps);
}

/**
 * Return a string identifying the focused control, suitable to use to restore
 * focus later...
 * @param w The VWindow to consider.
 * @return A string representing the focus.
 */
function GetFocusedControl(w)
{
  var tab =
    {
      tree: w.tree,
      hide_unimportant: w.hide_unimportant,
      draw_tree: w.draw_tree,
      var_important: w.var_important,
      ivbox: w.ivbox
    };
  var allR = "," + w.ivbox.selectionStart + "-" + w.ivbox.selectionEnd;

  for (name in tab)
  {
    if (tab[name].isfocused)
      return name + allR;
  }
  return "nofocus" + allR;
}

/**
 * Sets the focus based on a string from GetFocusedControl.
 * @param w The VWindow to consider.
 * @param f The focus string returned by GetFocusedControl.
 */
function SetFocusedControl(w, f)
{
  var allf = f.split(',');

  var sel = w.win.getSelection();
  sel.removeAllRanges();

  var p = allf[1].split('-');
  w.ivbox.selectionStart = p[0];
  w.ivbox.selectionEnd = p[1];

  var tab =
    {
      tree: w.tree,
      hide_unimportant: w.hide_unimportant,
      draw_tree: w.draw_tree,
      var_important: w.var_important,
      ivbox: w.ivbox
    };
  if (tab[allf[0]] == null)
    return;

  tab[allf[0]].isfocused = true;
  var x = tab[allf[0]];
  setTimeout(function() {x.focus();}, 1);
}

/**
 * Callback for when the variable view is being changed to something else.
 */
function FinishVariables(destroy)
{
  if (this.activeModel != null)
  {
    if (this.hide_unimportant == null)
      this.activeModel.xmlmi.hideUnimportant = false;
    else
      this.activeModel.xmlmi.hideUnimportant = this.hide_unimportant.hasAttribute("checked");
    
    if (this.draw_tree == null)
      this.activeModel.xmlmi.drawTree = true;
    else
      this.activeModel.xmlmi.drawTree = this.draw_tree.hasAttribute("checked");
    
    this.activeModel.xmlmi.lastTreeState = StringifyTreeState(this.tree);
    var fc = GetFocusedControl(this);
    this.activeModel.xmlmi.focusWhere = fc;
    if (this.modlev)
    {
      this.activeModel.xmlmi.lastImportantState = this.modlev.stringifyImportantState();
      if (destroy)
        this.modlev.destroy();
    }
  }
}

/**
 * Set the currently active view tree.
 * @param w The window which stores the state, and sources.
 */
function SetActiveVariableTree(w)
{
  var rsa;

  var hu = (w.hide_unimportant.getAttribute("checked") == "true");
  var dt = (w.draw_tree.getAttribute("checked") == "true");
  
  if (hu && dt)
    rsa = w.rdfsource_important_tree;
  else if (hu)
    rsa = w.rdfsource_important_flat;
  else if (dt)
    rsa = w.rdfsource_everything_tree;
  else
    rsa = w.rdfsource_everything_flat;

  if (w.rdfsource_active == rsa)
    return;

  if (w.rdfsource_active != null)
    w.tree.database.RemoveDataSource(w.rdfsource_active);

  w.rdfsource_active = rsa;
  w.tree.database.AddDataSource(w.rdfsource_active);
  w.tree.builder.rebuild();
}

/**
 * Called whenever the variables view needs to update the model.
 * @param m The new ModelInstance.
 */
function VariablesUpdateModel(m)
{
  this.variables_Finish(true);

  this.activeModel = m;
  this.activeModel.rwin = window;
  this.modelNameLabel.setAttribute("value", m.name);

  this.rdfsource_everything_tree =
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);
  this.rdfsource_important_tree =
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);
  this.rdfsource_everything_flat =
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);
  this.rdfsource_important_flat =
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);


  this.rdfindex =
    window.rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableIndex");

  this.rdfcu = window.rdfcu;
  
  this.modlev = new ModelLevel(m, window.rdfserv,
                               this.rdfsource_everything_tree,
                               this.rdfsource_important_tree,
                               this.rdfsource_everything_flat,
                               this.rdfsource_important_flat,
                               window.rdfcu);
  var allvars = new Object();
  var numvars = new Object();
  m.cellml.getAllVariables(Components.interfaces.mozICellML.NO_IN_VARIABLES,
                           allvars, numvars);

  numvars = numvars.value;
  var cellmlvar, i;
  this.varlevs = [];
  for (i = 0; i < numvars; i++)
  {
    cellmlvar = allvars.value[i];
    this.varlevs[i] = new VariableLevel(this.modlev, cellmlvar.name, cellmlvar, i);
  }

  // alert(this.tree);
  this.tree.window = this;
  this.ivbox.window = this;

  if (this.activeModel.xmlmi.hideUnimportant)
    this.hide_unimportant.setAttribute("checked", "true");
  else
    this.hide_unimportant.removeAttribute("checked");

  if (this.activeModel.xmlmi.drawTree)
    this.draw_tree.setAttribute("checked", "true");
  else
    this.draw_tree.removeAttribute("checked");

  SetActiveVariableTree(this);

  if (this.activeModel.xmlmi.lastImportantState != null)
  {
    this.modlev.restoreImportantState(this.activeModel.xmlmi.lastImportantState);
  }
  if (this.activeModel.xmlmi.lastTreeState != null)
  {
    LoadTreeState(this.tree, this.activeModel.xmlmi.lastTreeState);
    SelectVariable(this.tree);
  }

  if (this.activeModel.xmlmi.focusWhere != null)
    SetFocusedControl(this, this.activeModel.xmlmi.focusWhere);
}

/**
 * Called after a variable has been selected.
 * @param tc The tree control involved.
 */
function SelectVariable(tc)
{
  tc = tc.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  var idx = GetRDFIndexFromTree(tc.window, tc);
  var hb1 = tc.parentNode.getElementsByTagName("hbox").item(1);
  var important = hb1.getElementsByTagName("checkbox").item(2);
  var hbr = hb1.getElementsByTagName("hbox").item(0);
  var lbs = hb1.getElementsByTagName("label");
  var lbname = lbs.item(0);
  var lbunits = lbs.item(2);
  var tname = hb1.getElementsByTagNameNS("http://www.w3.org/1999/xhtml",
                                         "input").item(0);

  if (idx == -1)
  {
    tname.setAttribute("disabled", "true");
    tname.value = "";
    lbname.setAttribute("value", "");
    lbunits.setAttribute("value", "");
    important.setAttribute("disabled", "true");
    return;
  }

  important.removeAttribute("disabled");

  var vlev = tc.window.varlevs[idx];

  var value = "";
  try
  {
    value = tc.window.activeModel.getInitialValue(vlev.implev.number,
                                                  vlev.complev.name,
                                                  vlev.name);
  }
  catch (e)
  {
  }

  if ((vlev.cellmlvar.type ==
       Components.interfaces.mozICellMLVariable.BOUND) ||
      (vlev.cellmlvar.type ==
       Components.interfaces.mozICellMLVariable.SIMPLE))
    tname.setAttribute("disabled", "true");
  else
    tname.removeAttribute("disabled");

  tname.value = value;
  lbname.setAttribute("value", vlev.name);
  lbunits.setAttribute("value", vlev.cellmlvar.units);
  important.setAttribute("disabled", "false");
  if (vlev.important)
    important.setAttribute("checked", "true");
  else
    important.removeAttribute("checked");
}

/**
 * Called whenever the "Important" checkbox is clicked.
 * @param cb The checkbox that was changed.
 */
function ChangeImportant(cb)
{
  var tc = cb.parentNode.parentNode.getElementsByTagName("tree")
            .item(0);
  tc = tc.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  var idx = GetRDFIndexFromTree(tc.window, tc);
  if (idx == -1)
    return;

  tc.window.varlevs[idx].setImportant((cb.hasAttribute("checked")));
}

/**
 * Called whenever the "Hide Unimportant" checkbox is clicked.
 * @param cb The checkbox that was changed.
 */
function ChangeHideUnimportant(cb)
{
  SetActiveVariableTree(cb.parentNode.parentNode.getElementsByTagName("tree")
                          .item(0).window);
}

/**
 * Called whenever the "Draw as tree" checkbox is clicked.
 * @param cb The checkbox that was changed.
 */
function ChangeTreeDraw(cb)
{
  var tc = cb.parentNode.parentNode.getElementsByTagName("tree").item(0);
  SetActiveVariableTree(cb.parentNode.parentNode.getElementsByTagName("tree")
                          .item(0).window);
}

/**
 * Called whenever the initial condition is modified.
 * @param tb The textbox in which the initial condition was modified.
 */
function InitialConditionKey(tb, e)
{
  if (e.charCode == 0)
    return;
  var w = tb.window;

  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    // Time to clone the active model to make a new model, which becomes the
    // new active model...
    var omod = w.activeModel;
    var nmod = w.activeModel.cloneInstance();

    // Temporarily unlock this window...
    var oldlock = w.lockon.getAttribute("checked");
    w.lockon.setAttribute("checked", "false");
    var valt = tb.value;
    ModelSwitch(w.win.top, nmod);
    w.lockon.setAttribute("checked", oldlock);
  }
  // At this point, w.activeModel is dirty, and so we can safely modify it.
}

function ChangeInitialCondition(tb, e)
{
  var w = tb.window;

  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    /* They must have used a non-key means of editing(e.g. clipboard). This
     * hack will probably get the selection wrong, but at least it should
     * get the text entered correct.
     */
    var x = tb.value;
    InitialConditionKey(tb, e);
    tb.value = x;
  }

  if (!w.activeModel.dirty)
  {
    alert("The 'impossible' happened: An initial condition in a non-dirty\n "+
          " model was modified. This is a bug.\n" +
          "What should have happened is that the non-dirty model should have \n" +
          "been cloned to produce a dirty model when a keypress was detected.");
    return;
  }

  // Find the selected variable...
  var idx = GetRDFIndexFromTree(w, w.tree);
  var vlev = w.varlevs[idx];

  w.activeModel.setInitialValue(vlev.implev.number, vlev.complev.name,
                                vlev.name, parseFloat(tb.value));
}

function SaveAsCellML(b)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var model = b.parentNode.parentNode.vwin.activeModel;
  if (model == null)
  {
    GetStatusBar().setAttribute("label", "Select a model first.");
    return;
  }
  try
  {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(nsIFilePicker);
    fp.init(window, strb.getString("chooseSaveFilename"),
            nsIFilePicker.modeSave);
    fp.appendFilter(strb.getString("allCellML"), "*.cml; *.xml");
    fp.appendFilter("CML", "*.cml");
    fp.appendFilter("XML", "*.xml");
    fp.appendFilter(strb.getString("allFiles"), "*");
    if (fp.show() == nsIFilePicker.returnCancel)
      return;
    var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                           .createInstance(Components.interfaces.nsIFileOutputStream);
    var serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"]
                               .createInstance(Components.interfaces.nsIDOMSerializer);
    stream.init(
                fp.file,
                /* PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE */
                42,
                /*0644*/420,
                0
               );
    var savedom = model.xmlmi.cellml.
      QueryInterface(Components.interfaces.mozICellMLLocal).DOM;

    var backup_ivs = [];
    var cellml = model.xmlmi.cellml;

    if (model.xmlmi != model &&
        model.initialValues != null)
    {
      var imp, comp, varn;
      for (imp in model.initialValues)
      {
        for (comp in model.initialValues[imp])
        {
          for (varn in model.initialValues[imp][comp])
          {
            var newiv = model.initialValues[imp][comp][varn].value;
            var cv = cellml.getVariable(imp, comp, varn);
            var iv = cv.initialValue;
            if (iv != newiv)
            {
              if (backup_ivs[imp] == null)
                backup_ivs[imp] = {};
              if (backup_ivs[imp][comp] == null)
                backup_ivs[imp][comp] = {};
              backup_ivs[imp][comp][varn] = iv;
              cv.initialValue = newiv;
            }
          }
        }
      }
    }
    serializer.serializeToStream(savedom, stream, "UTF8");
    // Now restore the model...
    var imp, comp, varn;
    for (imp in backup_ivs)
    {
      for (comp in backup_ivs[imp])
      {
        for (varn in backup_ivs[imp][comp])
        {
          cellml.getVariable(imp, comp, varn).initialValue =
            backup_ivs[imp][comp][varn];
        }
      }
    }
    
  }
  catch (ex)
  {
    alert(ex);
  }
}
