#!/usr/bin/python
# Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
# http://JabXPCOM.sunsite.dk/

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

# I have only made minor changes to the original JabXPCOM test program
# to make it work with accessing the XPCOM components over CORBA.

import xpcom.components

JABBER_SERVER = raw_input('Jabber server: ')
JABBER_USER = raw_input('Username: ')
JABBER_PASSWORD = raw_input('Password: ')


CORBA_ORBPROXY_CONTRACTID = '@xpcom2corba.sunsite.dk/CORBA/ORBProxy;1'

JABBERSESSION_CONTRACTID = '@JabXPCOM.sunsite.dk/Session;1'
TCPSTREAM_CONTRACTID = '@JabXPCOM.sunsite.dk/TcpStream;1'

jabISession = xpcom.components.interfaces.jabISession
jabIPresence = xpcom.components.interfaces.jabIPresence
jabIInfoQuery = xpcom.components.interfaces.jabIInfoQuery

orbproxy = xpcom.components.classes[CORBA_ORBPROXY_CONTRACTID].createInstance(xpcom.components.interfaces.CORBA_IORBProxy)

print 'trying to load jabxpcom module'
if not orbproxy.load_module('jabxpcom'):
    print 'failed...'

print 'trying to load CosNaming module'
if not orbproxy.load_module('CosNaming'):
    print 'failed...'

nsctx = orbproxy.resolve_initial_references('NameService')
nsctx.queryInterface(xpcom.components.interfaces.CosNaming_INamingContextExt)

comp_mgr = nsctx.resolve_str('ComponentManager')
comp_mgr.queryInterface(xpcom.components.interfaces.XPCOM_IComponentManager)

jab_session = comp_mgr.create_instance(JABBERSESSION_CONTRACTID)
jab_session.queryInterface(jabISession)


class JabberSessionEventHandler:
    _com_interfaces_ = [xpcom.components.interfaces.jabISessionEvtConnected,
                        xpcom.components.interfaces.jabISessionEvtDisconnected,
                        xpcom.components.interfaces.jabISessionEvtAuthError,
                        xpcom.components.interfaces.jabISessionEvtIq,
                        xpcom.components.interfaces.jabISessionEvtMessage,
                        xpcom.components.interfaces.jabISessionEvtUnknownPacket,
                        xpcom.components.interfaces.jabISessionEvtIqVersion,
                        xpcom.components.interfaces.jabISessionEvtIqLast,
                        xpcom.components.interfaces.jabISessionEvtPresence,
                        xpcom.components.interfaces.jabISessionEvtPresenceRequest]

    def __init__(self):
        pass

    def onConnected(self, tag):
        print "connected, id:", tag.getAttrib("id"), tag.getXML()

        # request agents list from server
        iq_agents = jab_session.createInfoQuery("", jabIInfoQuery.iqtGet)
        iq_agents.addQuery("agents")
        jab_session.sendPacket(iq_agents)

        my_presence = jab_session.createMyPresence(jabIPresence.ptAvailable,
                                                   jabIPresence.stOnline,
                                                   "", 0)
        jab_session.sendPacket(my_presence)

    def onDisconnected(self):
        print "disconnected"

    def onAuthError(self, code, data):
        print "authError", code, data
        jab_session.disconnect()

    def onIq(self, tag):
        print "iq", tag.getXML()

    def onMessage(self, message):
        print "message", message.sender, message.to, message.body
        envelope = message.getExtension('envelope')
        if envelope:
            print 'envelope from', envelope.getTag('from').getAttrib('jid')

        delay = message.getExtension('delay')
        if delay:
            print 'delay', delay.getAttrib('stamp')

        jab_session.sendPacket(message.reply('Hello world'))

    def onUnknownPacket(self, tag):
        print "unknownPacket", tag.getXML()

    def onIqVersion(self):
        print "iqVersion"
        return ('jabxpcom', '0.20', '')

    def onIqLast(self):
        print "iqLast"
        return ''

    def onPresence(self, presence, type):
        print "presence", presence.sender, type

    def onPresenceRequest(self, presence):
        print "presenceRequest", presence.sender

        # accept presence request
        presence_reply = presence.reply(jabIPresence.ptSubscribed)
        jab_session.sendPacket(presence_reply)


# register event handlers
event_handler = JabberSessionEventHandler()

jab_session.connectEvtConnected(event_handler)
jab_session.connectEvtDisconnected(event_handler)
jab_session.connectEvtAuthError(event_handler)
jab_session.connectEvtIq(event_handler)
jab_session.connectEvtMessage(event_handler)
jab_session.connectEvtUnknownPacket(event_handler)
jab_session.connectEvtIqVersion(event_handler)
jab_session.connectEvtIqLast(event_handler)
jab_session.connectEvtPresence(event_handler)
jab_session.connectEvtPresenceRequest(event_handler)

jab_session_input = jab_session.inputStream


# remote tcp stream
tcp_stream = comp_mgr.create_instance(TCPSTREAM_CONTRACTID)
tcp_stream.queryInterface(xpcom.components.interfaces.jabIDataStream)

tcp_stream.queryInterface(xpcom.components.interfaces.jabIStreamClientConnector)
tcp_stream.queryInterface(xpcom.components.interfaces.jabIStreamOutputConnector)


tcp_stream.connect_client(jab_session_input)
tcp_stream.connect_output(JABBER_SERVER + ':5222')

jab_session.outputStream = tcp_stream


print "connecting..."
jab_session.connect(JABBER_SERVER, jabISession.atAutoAuth,
                    JABBER_USER, "jabxpcom", JABBER_PASSWORD, 0)


import time
time.sleep(600)
