/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */

#include "corba_locator.h"

#include "corba_proxy.h"

#include <pratom.h>

#include <algorithm>
#include <cstring>


CORBA_Proxy_Locator::CORBA_Proxy_Locator()
  : oid_counter_(0)
{
}


CORBA_Proxy_Locator::~CORBA_Proxy_Locator()
{
}


PortableServer::Servant
CORBA_Proxy_Locator::preinvoke(const PortableServer::ObjectId &oid,
			       PortableServer::POA_ptr adapter,
			       const char *operation,
			       PortableServer::ServantLocator::Cookie &the_cookie)
{
  CORBA::String_var oid_string = PortableServer::ObjectId_to_string(oid);
  Oid_Map::const_iterator oid_iter = oids_.find(oid_string);

  if (oid_iter != oids_.end())
  {
    CORBA_Proxy_Object *proxy_obj = oid_iter->second;

    if (!::strcmp(operation, "_is_a") ||
	!::strcmp(operation, "_non_existent") ||
	!::strcmp(operation, "add_ref") ||
	!::strcmp(operation, "release_ref"))
    {
      // internally handle these pseudo-operations...
      return proxy_obj->default_servant();
    }
    else
    {
      PortableServer::Servant servant =
	proxy_obj->servant_for_operation(operation);

      return servant;
    }
  }

  // oh dear, couldn't find servant
  return NULL;
}


void
CORBA_Proxy_Locator::postinvoke(const PortableServer::ObjectId &oid,
				PortableServer::POA_ptr adapter,
				const char *operation,
				PortableServer::ServantLocator::Cookie the_cookie,
				PortableServer::Servant the_servant)
{
}


const char *CORBA_Proxy_Locator::register_proxy(CORBA_Proxy_Object *proxy_obj)
{
  PRInt32 oid_int = PR_AtomicIncrement(&oid_counter_);

  PRUint8 oid_bytes[4];
  int oid_length = 0;

  // take care not to include any binary 0's in the oid string
  while (oid_int)
  {
    oid_bytes[oid_length] = (oid_int % 255) + 1;
    oid_int /= 255;

    oid_length++;
  }

  char *oid = new char[oid_length + 1];

  std::copy(oid_bytes, oid_bytes + oid_length, oid);
  oid[oid_length] = 0;

  oids_[oid] = proxy_obj;

  return oid;
}


void CORBA_Proxy_Locator::unregister_proxy(const char *oid_str)
{
  const Oid_Map::iterator oid_iter = oids_.find(oid_str);

  if (oid_iter != oids_.end())
  {
    oids_.erase(oid_iter);

    delete [] const_cast<char*>(oid_str);
  }
  else
  {
    // TODO: hmm... trying to delete non-existent object id
  }
}
