/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */
#ifndef HEADER__CORBA_PROXY__INCLUDED
#define HEADER__CORBA_PROXY__INCLUDED

#include "cstring_util.h"

#include <nsISupports.h>
#include <nsCOMPtr.h>

#include <xpcom.hh>

#include <map>


class CORBA_Proxy_Servant;


class CORBA_Proxy_Factory
{
 protected:
  CORBA_Proxy_Factory(const char *repo_id, const CORBA_Proxy_Factory *factory);

 public:
  virtual ~CORBA_Proxy_Factory();


  virtual CORBA_Proxy_Servant *GetProxy(nsISupports *obj) const = 0;

  static CORBA_Proxy_Servant *GetProxy(const char *repo_id,
				       nsISupports *obj);


 private:
  typedef std::map<const char *, const CORBA_Proxy_Factory *,
		   cstring_Less> Factory_Map;

  static Factory_Map *factories_;

  Factory_Map::iterator factory_iter_;
};


class CORBA_Proxy_Object
  : public POA_XPCOM::IObject
{
 public:
  CORBA_Proxy_Object(nsISupports *xpcom_obj);

  virtual ~CORBA_Proxy_Object();


  virtual CORBA::Boolean _is_a(const char *type);

  virtual CORBA::Boolean _non_existent();

  virtual void add_ref();

  virtual void release_ref();


  inline PortableServer::Servant default_servant()
  { return this; }

  PortableServer::Servant servant_for_operation(const char *operation);


  bool add_repoid(const char *type);

  void oid(const char *oid_str);


 protected:
  void add_operations(const char **operation_iter,
		      CORBA_Proxy_Servant *proxy_servant);

  void add_operations_recursive(const char **repo_iter);


 private:
  nsCOMPtr<nsISupports> xpcom_obj_;

  PRInt32 ref_count_;

  /// memory owned by Locator map
  const char *oid_;


  // map method name to servant
  typedef std::map<const char *, CORBA_Proxy_Servant *,
		   cstring_Less> Operation_Map;

  Operation_Map operations_;


  // map repository id to interface pointer (nsISupports *)
  typedef std::map<const char *, CORBA_Proxy_Servant *,
		   cstring_Less> RepoId_Map;

  RepoId_Map repo_ids_;
};

#endif
