/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */

#include <CORBA.h>

#include "corba_locator.h"
#include "corba_poa.h"
#include "corba_proxy.h"
#include "xpcom_object.h"

#include <xpcom4corba.hh>
#include <Naming.hh>

#include <nsCOMPtr.h>
#include <nsIComponentLoader.h>
#include <nsIComponentManager.h>
#include <nsISupports.h>
#include <nsMemory.h>
#include <nsXPCOM.h>
#include <nspr.h>

#include <iostream>
#include <string>


class XPCOM_ComponentManager_Impl
  : public POA_XPCOM::ComponentManager,
    public PortableServer::RefCountServantBase
{
 public:
  XPCOM_ComponentManager_Impl()
  { }

  virtual ~XPCOM_ComponentManager_Impl()
  { }


  XPCOM::IObject_ptr create_instance(const char *contract_id);
};


XPCOM::IObject_ptr
XPCOM_ComponentManager_Impl::create_instance(const char *contract_id)
{
  nsresult rv;
  nsCOMPtr<nsISupports> xpcom_obj = do_CreateInstance(contract_id, &rv);
  if (NS_FAILED(rv))
  {
    std::cerr << "ERROR: Cannot create instance of component \""
	      << contract_id << "\" [" << rv << "]."
	      << std::endl;
  }

  XPCOM::IObject* dummy = (XPCOM::IObject*)NULL;

  nsISupports* xpcom_obj_raw = xpcom_obj;
  XPCOM::IObject_ptr corba_obj =
    the_corba_poa->register_object(xpcom_obj_raw, dummy);

  return corba_obj;
}


int main(int argc, char *argv[])
{
  nsresult rv;

  // Initialize XPCOM
  rv = NS_InitXPCOM2(nsnull, nsnull, nsnull);
  if (NS_FAILED(rv))
  {
    std::cerr << "ERROR: XPCOM initialization error [" << rv << "]."
	      << std::endl;

    return -1;
  }

  // nsComponentManager::AutoRegister(nsIComponentLoader::Startup, nsnull);


  try
  {
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
    CORBA::Object_var poa_obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poa_obj);

    the_corba_poa = new CORBA_POA();
    the_corba_poa->init(argc, argv);


    XPCOM_ComponentManager_Impl *compman_i = new XPCOM_ComponentManager_Impl();

    PortableServer::ObjectId_var compman_id = poa->activate_object(compman_i);

    XPCOM::ComponentManager_var compman = compman_i->_this();
    CORBA::String_var compman_ior(orb->object_to_string(compman));
    std::cout << compman_ior << std::endl;


    CORBA::Object_var nctx_obj =
      orb->resolve_initial_references("NameService");
    CosNaming::NamingContextExt_var nctx =
      CosNaming::NamingContextExt::_narrow(nctx_obj);

    CosNaming::Name_var name = nctx->to_name("ComponentManager");
    nctx->rebind(name, compman);


    for (int i = 1; i < argc; i++)
    {
      std::string filename("./xpcorba_");
      filename += argv[i];
      filename += ".so";

      bool opened = (PR_LoadLibrary(filename.c_str()) != NULL);
      std::cout << "loading of module \"" << filename << "\": "
		<< opened << std::endl;
    }


    orb->run();

    orb->destroy();
  }
  catch(CORBA::SystemException&)
  {
    std::cerr << "Caught CORBA::SystemException." << std::endl;
  }
  catch(CORBA::Exception&)
  {
    std::cerr << "Caught CORBA::Exception." << std::endl;
  }
  catch(...)
  {
    std::cerr << "Caught unknown exception." << std::endl;
  }

  return 0;
}
