/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */

#include "xpcom_proxy.h"


bool nsIID_Less::operator()(const nsIID *x, const nsIID *y) const
{
  // looks a bit like a hack but should be fairly efficient
  return (x->m0 < y->m0) ||
    ((x->m0 == y->m0) &&
     ((((PRUint32 *) &x->m0)[1] < ((PRUint32 *) &y->m0)[1]) ||
      ((((PRUint32 *) &x->m0)[1] == ((PRUint32 *) &y->m0)[1]) &&
       ((((PRUint32 *) &x->m0)[2] < ((PRUint32 *) &y->m0)[2]) ||
	((((PRUint32 *) &x->m0)[2] == ((PRUint32 *) &y->m0)[2]) &&
	 ((((PRUint32 *) &x->m0)[3] < ((PRUint32 *) &y->m0)[3])))))));
}


XPCOM_Proxy_Factory::Factory_Map *XPCOM_Proxy_Factory::factories_ = 0;


XPCOM_Proxy_Factory::XPCOM_Proxy_Factory(const nsIID &iid,
					 const XPCOM_Proxy_Factory *factory)
{
  // Hmm, static initialization doesn't work, therefore we have to use
  // a singleton here
  if (!factories_)
    factories_ = new Factory_Map();

  factory_iter_ =
    factories_->insert(Factory_Map::value_type(&iid, factory)).first;
}

XPCOM_Proxy_Factory::~XPCOM_Proxy_Factory()
{
  if (factories_)
  {
    factories_->erase(factory_iter_);

    if (factories_->size() == 0)
    {
      delete factories_;
      factories_ = 0;
    }
  }
}


CORBA_IObject *
XPCOM_Proxy_Factory::GetProxy(const nsIID &iid,
			      CORBA::Object_ptr obj, PRInt32 *ref_count)
{
  Factory_Map::const_iterator iter = factories_->find(&iid);
  if (iter != factories_->end())
  {
    return iter->second->GetProxy(obj, ref_count);
  }

  return 0;
}
